package codeline;

import processor.ControlFlow;
import processor.Memory;
import processor.StackFrame;
import processor.Memory.ReadOnlyMemory;
import processor.Memory.Visibility;
import processor.StackFrame.FrameType;

/**
 * Yey, IF is a LOOP.
 * @author kolja
 *
 */
public abstract class IfLoop extends CodeLine {

	@Override
	public ControlFlow runForward(Memory m) {
		// Prfe ob die Kindknoten bereits ausgefhrt wurden
		if( m.isDefined( "line_" + lineId + "_inside", Visibility.LOCAL ) )
		{ // Wurde bereits ausgefhrt
			StackFrame old = m.removeFrame();
			actions.push( (Memory mem) -> {
				mem.addFrame( old );
			} );
            return new ControlFlow( ControlFlow.STEP_OVER ); // Springe zum nachfolgenden Knoten
		}
		else
		{ // Wurde noch nicht ausgefhrt
            if( condition( m.createReadOnlyMemory() ) ) { // Prfe ob die Bedingung erfllt ist
            	m.addFrame( new StackFrame( FrameType.LOOP ) );
            	m.declare( "line_" + lineId + "_inside", true, Visibility.LOCAL );
            	actions.push( (Memory mem) -> {
            		mem.removeFrame();
            	} );
            	return new ControlFlow( ControlFlow.STEP_INTO ); // Springe in das If rein
            } else {
            	actions.push( (Memory mem) -> {} );
                return new ControlFlow( ControlFlow.STEP_OVER ); // Springe zum nachfolgenden Knoten
            }
		}
	}

    /**
     * the exit condition of the loop
     * @param m the memory in which the variables are stored
     * @return false if the loop is over
     */
	protected abstract boolean condition( ReadOnlyMemory m );
}
